/*******************************************************************/ /**
* @file algorithm.cpp
*
* @brief Implementation of the optimalization method
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.18 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#include "algorithm.h"

#include <sstream>

namespace irafm {

std::string finalStatusToStr(FinalStatus status) {
   switch (status) {

      case fsUnknown:
         return "Unknown";

      case fsSuccess:
         return "Success";

      case fsNoSuccess:
         return "No success";

      case fsItExceeded:
         return "Number of iterations exceeded";
   }

   return "Status Error";
}

/****************************************************************/ /**
* This method is called after each new population generation to determine
* if new population satisfies required limitations. At least one of the
* following conditions has to be satisfied to stop the algorithm:
*
* -# if objective function value of the best member of population is lower
*    than precision limit.
* -# if number of performed iterations is out of iteration limit.
*
* @return logical value deciding about termination of optimization process
*
* @todo Pridat varianty ukoncovaci podminky
*	- limit poctu iteraci /done/
*		@param limit iteraci
*	- limit poctu vyhodnoceni obj. fce
*		@param limit vyhodnoceni
*	- dlouho nedochazi ke zmene
*		@param pocet iteraci bez zlepseni nejlepsiho clena populace
*	- rozdil mezi nejlepsim a nejhorsim dosahne pozadovane hodnoty /done/
*		@param max. rozdil
*	- dosazeni pozadovane hodnoty v pripade, ze je znama /done/
*		@param certifikovana hodnota
*		@param pozadovana presnost
*/
bool Algorithm::stopCondition() {
   /// TODO Zadefinovat parametr pro stopLimit, misto magic number 0.000001
   if (population_.worst()->objectiveFunction() - population_.best()->objectiveFunction() < 0.0000001)
      return true;

   return getNumberOfIterations() > iterationsLimit_;
}

/****************************************************************/ /**
* Main method performing evolutionary style optimization. Initial
* population is transformed into new populations (in loop) until
* the stop condition is fulfilled.
*
* @return The best population member representing the result of
*   optimization.
*/
ptrPopulationMember Algorithm::optimize(
   ptrProblem aProblem, ///<: specification of problem to optimize
   ptrReporter reporter ///<: instance of progress and results reporter
) {
   // store problem for internal usage
   problem_ = aProblem;

   // initialize random number generator
   srand(time(NULL));

   PopulationMember::resetCounter();

   numberOfIterations_ = 0;

   // generate initial population
   population_ = problem_->initialPopulation();

   while (!stopCondition()) {
      reporter->beginIteration(this);

      computeNewPopulation();

      ++numberOfIterations_;

      reporter->endIteration(this);
#ifdef DEBUG
      std::cout
         << "iteration: " << numberOfIterations_
         << ", best member value: "
         << population_.best()->objectiveFunction() << std::endl;
#endif
   }

   return population_.best();
}

std::string Algorithm::getName() {
   return name_ + "<" + memberGenerator_->getName() + ">" + getSettingsStr();
}

std::string Algorithm::getSettingsStr() {
   std::ostringstream params;

   params
   << "["
   << "maxIterations: " << iterationsLimit_
   << ", reqPrecision: " << precisionLimit_
   << "]";

   return params.str();
}

/****************************************************************/ /**
* Provides infromation about success / failure of the optimization
*/
FinalStatus Algorithm::getStatus() {
   if (population_.best()->objectiveFunction() < precisionLimit_) {
      return fsSuccess;
   }

   if (getNumberOfIterations() > iterationsLimit_) {
      return fsItExceeded;
   }

   return fsNoSuccess;
}

/****************************************************************/ /**
* Text style report with statistics of performed optimization. The
* report contains:
*   - %Problem name (e.g. Rastrigin function minimalization)
*   - %Algorithm name (e.g. comp debr18)
*   - Best Member in %Population with function value
*   - Number of Iterations
*   - Number of Objective Function Evaluations
*   - Status - one of the following values:
*       - Unknown
*       - Success
*       - Number of iterations exceeded
*       - Status Error
*
Sample report:
@verbatim
Problem: Rastrigin function
Algorithm: Differential Evolution<comp der9>
Best Member in Population: [(-1.83602e-005, 6.25492e-006, 4.01758e-006): 4.72523e-007]
Number of iterations: 326
Number of Objective Function Evaluations: 16350
Status: Success
@endverbatim
*
* @return string containing the report.
*/
std::string Algorithm::report() {
   std::ostringstream result;

   result
   << "Algorithm: " << getName() << std::endl
   << "Task: " << problem_->name << std::endl
   << "Best Member in Population: " << population_.best()->print() << std::endl
   << "Number of iterations: " << getNumberOfIterations() << std::endl
   << "Number of Objective Function Evaluations: "
   << PopulationMember::numberOfCalls() << std::endl
   << "Status: " << finalStatusToStr(getStatus());

   return result.str();
}

}

// $Log: algorithm.cpp,v $
// Revision 1.18  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.17  2009/09/15 13:21:23  victor
// +TODO entries
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.16  2009/05/04 08:43:25  victor
// # Zmena ukoncovaci podminky
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.15  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.14  2009/03/27 19:36:58  victor
// Satisfying linux compatibility
//
// Revision 1.13  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.12  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
// Revision 1.11  2008/04/07 09:12:17  victor
// Managing TODO items
//
// Revision 1.10  2008/01/21 11:52:48  victor
// Documenting the source code
//
// Revision 1.9  2007/12/13 15:13:03  victor
// + Algorithm final result status
//
// Revision 1.8  2007/12/04 15:03:22  victor
// + DEBUG information for testing
//
// Revision 1.7  2007/11/26 14:21:42  victor
// # changed report order
//
// Revision 1.6  2007/11/26 12:33:44  victor
// + Member Generator given a name for identification in reports
//
// Revision 1.5  2007/11/26 12:15:09  victor
// + Algorithm given a name for identification in reports
//
// Revision 1.4  2007/11/26 09:36:27  victor
// + Problem given a name for identification in reports
//
// Revision 1.3  2007/11/21 09:05:28  victor
// + Counting evaluation of objective function
// + Caching value of objective function
//
// Revision 1.2  2007/11/20 11:59:42  victor
// Refactorings: memberGenerator_, stopCondition()
// Added: std::string report()
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
