/*******************************************************************/ /**
* @file algorithm.h
*
* @brief Specification of the optimalization method
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.14 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef ALGORITHM_H
#define ALGORITHM_H

// STL
#include <string>

#include <irafm/smart_ptr.h>

namespace irafm {

class Algorithm;

/// smart pointer to Algorithm
typedef yasper::ptr<Algorithm> ptrAlgorithm;

}

#include "member_generator.h"
#include "problem.h"
#include "reporter.h"

namespace irafm {

/// Encapsulates the possible final states after optimization
enum FinalStatus {
   fsUnknown,   ///<: unspecified error during optimization
   fsSuccess,   ///<: optimization was successful
   fsNoSuccess, ///<: optimization was not successful but finished before iteration limit overflow
   fsItExceeded ///<: optimization failed due to iteration limit overflow
};

/// converts enumeration type to string representation
std::string finalStatusToStr(FinalStatus status);

/// Specifies the method of optimization.
class Algorithm {
public:
   /// Constructor
   Algorithm(
      std::string name, ///<: algorithm name
      ptrMemberGenerator memberGenerator, ///<: pointer to object generating population members
      size_t maxIterations = 10000, ///<: limit of iterations for stopping
      real minPrecision = 0.000001 ///<: precision limit for optimal solution
   ):
         name_(name), memberGenerator_(memberGenerator),
         iterationsLimit_(maxIterations), precisionLimit_(minPrecision) {}

   /// Destructor
   virtual ~Algorithm() {}

   /// main optimization method (entry point)
   virtual ptrPopulationMember optimize(ptrProblem aProblem, ptrReporter reporter);

   /// abstract method generating new population (must be defined in successor)
   virtual void computeNewPopulation() abstract;

   /// @return current number of performed iterations from starting of algorithm
   size_t getNumberOfIterations() {
      return numberOfIterations_;
   }

   /// @return the name of algorithm
   virtual std::string getName();

   /// @return string containing algorithm settings
   virtual std::string getSettingsStr();

   /// textual report with statistics of processed optimiztion
   std::string report();

   /// @return final state of optimization
   virtual FinalStatus getStatus();

protected:
   /// name of the algorithm
   std::string name_;

   /// new member generator
   ptrMemberGenerator memberGenerator_;

   /// limit for algorithm iterations
   size_t iterationsLimit_;

   /// limit for algorithm difference from global minimum
   real precisionLimit_;

   /// number of computed iterations
   size_t numberOfIterations_;

   /// instance of problem for optimization
   ptrProblem problem_;

   /// current population of members
   Population population_;

   /// condition to stop iteration process
   /** @todo Change method stopCondition to Policy class */
   virtual bool stopCondition();

   /// @return dimension of current solving problem
   size_t dimension() const {
      return problem_->dimension();
   }

};

}

#endif // ALGORITHM_H

// $Log: algorithm.h,v $
// Revision 1.14  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.13  2009/05/04 08:43:25  victor
// # Zmena ukoncovaci podminky
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.12  2009/04/21 12:19:56  victor
// # todo management
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.11  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.10  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.9  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
// Revision 1.8  2008/04/07 09:12:17  victor
// Managing TODO items
//
// Revision 1.7  2008/01/21 11:52:48  victor
// Documenting the source code
//
// Revision 1.6  2007/12/13 15:13:03  victor
// + Algorithm final result status
//
// Revision 1.5  2007/11/26 12:33:44  victor
// + Member Generator given a name for identification in reports
//
// Revision 1.4  2007/11/26 12:15:09  victor
// + Algorithm given a name for identification in reports
//
// Revision 1.3  2007/11/20 11:59:42  victor
// Refactorings: memberGenerator_, stopCondition()
// Added: std::string report()
//
// Revision 1.2  2007/11/19 15:14:25  victor
// Added Differential Evolution Algorithm
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
