/*******************************************************************/ /**
* @file benchmark_functions.cpp
*
* @brief Set of functions for benchmarking optimization algorithms
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.9 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#include "benchmark_functions.h"

namespace irafm {

real dejong1(Vector x) {
   // First De Jong fuction, <-5.12; 5.12>
   // glob. min f(0) = 0

   return (x*x).sum();
}

real rastrigin(Vector x) {
   // Rastrigin's fuction, <-5.12; 5.12>
   // glob. min f(0) = 0

   real A = 10;
   size_t dim = x.size();

   return A * dim + (x*x - A*cos(2 * M_PI * x)).sum();
}

real schwefel(Vector x) {
   // Schwefel's  function, <-500; 500>,
   // glob. min f(x_star)=-418.982887074 * d, x_star=[s,s,...,s], s=420.97

   size_t dim = x.size();

   // return (x * sin(sqrt(abs(x)))).sum() + 418.982887074 * dim;
   return (x * sin(sqrt(abs(x)))).sum() + 418.982887272433799807913601398 * dim;
}

real ackley(Vector x) {
   // Ackley's  function,, <-30; 30>,
   // glob. min f(0) = 0

   size_t dim = x.size();

   real a = -20.0 * exp(-0.02 * sqrt((x * x).sum() / dim));

   real b = -exp(cos(2 * M_PI * x).sum() / dim);

   return a + b + 20.0 + exp(1);

}

real griewank(Vector x) {
   // Griewank's  function, <-400; 400>,
   // glob. min f(0) = 0

   size_t dim = x.size();

   real a = (x * x).sum() / 4000.0 + 1;

   real b = 1;

   for (index_t i = 0; i < dim; i++) {
      b = b * cos(x[i] / sqrt(i + 1));
   }

   return a - b;
}

real rosen(Vector x) {
   // Rosenbrock's  function, <-2048; 2048>,
   // glob. min f(1) = 0

   size_t dim = x.size();

   real a = 0;

   for (index_t i = 0; i < dim - 1; i++) {
      real b = x[i] * x[i] - x[i+1];
      real c = (1 - x[i]);

      a = a +  100 * b * b + c * c;
   }

   return a;

}

}

// $Log: benchmark_functions.cpp,v $
// Revision 1.9  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.8  2009/05/04 08:41:43  victor
// # Oprava prehlednuteho deleni nulou
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.7  2009/04/30 09:06:46  victor
// + Griewang, Rosen (Radka)
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2009/04/21 12:19:02  victor
// # Renamed quadratic function to dejong1
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.3  2007/12/13 13:47:30  victor
// + ackley function
// # simplifying benchmark functions using vector arithmetics
//
// Revision 1.2  2007/11/26 14:20:33  victor
// + Function Object wrappers for benchmark functions
//
// Revision 1.1  2007/11/22 14:51:35  victor
// Initial revision
//
