/*******************************************************************/ /**
* @file competitive_member_generator.cpp
*
* @brief Set of competiting generators
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.13 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

// STL
#include <sstream>

#include "competitive_member_generator.h"
#include "de_member_generator.h"

namespace irafm {

void CompetitiveMemberGenerator::addGenerator(ptrMemberGenerator generator) {
   generators_.push_back(generator);

   hitRate_.setNumItems(generators_.size());
}

ptrMemberGenerator CompetitiveMemberGenerator::randomGenerator() {
   lastSelectedGeneratorIndex = hitRate_.randomItemIndex();

   return generators_[lastSelectedGeneratorIndex];
}

Vector CompetitiveMemberGenerator::generateTrialPoint(
   ptrProblem problem, Population &population
) {
   return randomGenerator()->generateTrialPoint(problem, population);
}

void CompetitiveMemberGenerator::reportSuccess(const bool success) {
   if (success) {
      hitRate_.inc(lastSelectedGeneratorIndex);
   }
}

ptrPopulationMember CompetitiveMemberGenerator::generateNewMember(
   ptrProblem problem, Population &population) {

   return randomGenerator()->generateNewMember(problem, population);
}

ptrPopulationMember CompetitiveMemberGenerator::generateNewMember(
   ptrProblem problem, Population &population, ptrPopulationMember member) {

   return randomGenerator()->generateNewMember(problem, population, member);
}

std::string CompetitiveMemberGenerator::getName() {
   return name_;
}

ptrMemberGenerator CompetitiveMemberGenerator::DER9() {
   // used values of parameter F
   real paramF[] = {0.5, 0.8, 1};

   // create all possible mutators
   std::vector<ptrMutation> mutations;

   for (index_t f = 0; f < 3; ++f) {
      mutations.push_back(new BestMutation(paramF[f]));
   }

   // used values of parameter C
   real paramC[] = {0, 0.5, 1};

   // create all possible crossovers
   std::vector<ptrCrossover> crossovers;

   for (index_t c = 0; c < 3; ++c) {
      crossovers.push_back(new BinomialCrossover(paramC[c]));
   }

   CompetitiveMemberGenerator *der9 = new CompetitiveMemberGenerator("CDE/3best/3bin/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         der9->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return der9;
}

ptrMemberGenerator CompetitiveMemberGenerator::DEBR18() {
   // used values of parameter F
   real paramF[] = {0.5, 0.8, 1};

   // create all possible mutators
   std::vector<ptrMutation> mutations;

   for (index_t f = 0; f < 3; ++f) {
      mutations.push_back(new BestMutation(paramF[f]));
      mutations.push_back(new RandMutation(paramF[f]));
   }

   // used values of parameter C
   real paramC[] = {0, 0.5, 1};

   // create all possible crossovers
   std::vector<ptrCrossover> crossovers;

   for (index_t c = 0; c < 3; ++c) {
      crossovers.push_back(new BinomialCrossover(paramC[c]));
   }

   CompetitiveMemberGenerator *debr18 = new CompetitiveMemberGenerator("CDE/3best,3rand/3bin/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         debr18->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return debr18;
}

ptrMemberGenerator CompetitiveMemberGenerator::Cbin9rl() {
   // array of mutators
   std::vector<ptrMutation> mutations;

   // create and add all possible mutators
   mutations.push_back(new Rand_RL_Mutation(0.5));
   mutations.push_back(new Rand_RL_Mutation(0.8));
   mutations.push_back(new Rand_RL_Mutation(1));

   // create all possible crossovers
   std::vector<ptrCrossover> crossovers;

   crossovers.push_back(new BinomialCrossover(0));
   crossovers.push_back(new BinomialCrossover(0.5));
   crossovers.push_back(new BinomialCrossover(1));

   CompetitiveMemberGenerator *cbin9rl = new CompetitiveMemberGenerator("CDE/3rl/3bin/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         cbin9rl->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return cbin9rl;
}

ptrMemberGenerator CompetitiveMemberGenerator::Cexp9rl(int dimension) {
   // array of mutators
   std::vector<ptrMutation> mutations;

   // create and add all possible mutators
   mutations.push_back(new Rand_RL_Mutation(0.5));
   mutations.push_back(new Rand_RL_Mutation(0.8));
   mutations.push_back(new Rand_RL_Mutation(1));

   real p2 = (1 + 1 / real(dimension)) / 2;
   real p1 = (1 / real(dimension) + p2) / 2;
   real p3 = (p2 + 1) / 2;

   // array of crossovers
   std::vector<ptrCrossover> crossovers;

   // create and add all possible crossovers
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p1, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p2, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p3, dimension)));

   CompetitiveMemberGenerator *cexp9rl = new CompetitiveMemberGenerator("CDE/3rl/3exp/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         cexp9rl->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return cexp9rl;
}

ptrMemberGenerator CompetitiveMemberGenerator::Cbin9exp9rl(int dimension) {
   // array of mutators
   std::vector<ptrMutation> mutations;

   // create and add all possible mutators F[0.5, 0.8, 1]
   mutations.push_back(new Rand_RL_Mutation(0.5));
   mutations.push_back(new Rand_RL_Mutation(0.8));
   mutations.push_back(new Rand_RL_Mutation(1));

   real p2 = (1 + 1 / real(dimension)) / 2;
   real p1 = (1 / real(dimension) + p2) / 2;
   real p3 = (p2 + 1) / 2;

   // array of crossovers
   std::vector<ptrCrossover> crossovers;

   // create and add all possible crossovers
   // 3 x binomial CR[0, 0.5, 1]
   crossovers.push_back(new BinomialCrossover(0));
   crossovers.push_back(new BinomialCrossover(0.5));
   crossovers.push_back(new BinomialCrossover(1));

   // 3 x exponential
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p1, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p2, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p3, dimension)));

   CompetitiveMemberGenerator *cbin9exp9rl = new CompetitiveMemberGenerator("CDE/3rl/3bin,3exp/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         cbin9exp9rl->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return cbin9exp9rl;
}

ptrMemberGenerator CompetitiveMemberGenerator::Cbin6exp6rl(int dimension) {
   // array of mutators
   std::vector<ptrMutation> mutations;

   // create and add all possible mutators F[0.5, 0.8]
   mutations.push_back(new Rand_RL_Mutation(0.5));
   mutations.push_back(new Rand_RL_Mutation(0.8));

   real p2 = (1 + 1 / real(dimension)) / 2;
   real p1 = (1 / real(dimension) + p2) / 2;
   real p3 = (p2 + 1) / 2;

   // array of crossovers
   std::vector<ptrCrossover> crossovers;

   // create and add all possible crossovers
   // 3 x binomial CR[0, 0.5, 1]
   crossovers.push_back(new BinomialCrossover(0));
   crossovers.push_back(new BinomialCrossover(0.5));
   crossovers.push_back(new BinomialCrossover(1));

   // 3 x exponential
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p1, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p2, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p3, dimension)));

   CompetitiveMemberGenerator *cbin6exp6rl = new CompetitiveMemberGenerator("CDE/2rl/3bin,3exp/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         cbin6exp6rl->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return cbin6exp6rl;
}

ptrMemberGenerator CompetitiveMemberGenerator::Cbin12exp8BeRan(int dimension) {
   // array of mutators
   std::vector<ptrMutation> mutations;

   // create and add all possible mutators F[0.5, 0.8]
   mutations.push_back(new BestMutation(0.5));
   mutations.push_back(new BestMutation(0.8));
   mutations.push_back(new RandMutation(0.5));
   mutations.push_back(new RandMutation(0.8));

   real p2 = (1 + 1 / real(dimension)) / 2;
   real p1 = (1 / real(dimension) + p2) / 2;

   // array of crossovers
   std::vector<ptrCrossover> crossovers;

   // create and add all possible crossovers
   // 3 x binomial CR[0, 0.5, 1]
   crossovers.push_back(new BinomialCrossover(0));
   crossovers.push_back(new BinomialCrossover(0.5));
   crossovers.push_back(new BinomialCrossover(1));

   // 2 x exponential
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p1, dimension)));
   crossovers.push_back(new ExponentialCrossover(ExponentialCrossover::findCR(p2, dimension)));

   CompetitiveMemberGenerator *Cbin12exp8BeRan = new CompetitiveMemberGenerator("CDE/2best,2rand/3bin,2exp/");

   // add all possible combinations of mutators and crossovers
   std::vector<ptrMutation>::iterator mutationIT;
   std::vector<ptrCrossover>::iterator crossoverIT;

   for (mutationIT = mutations.begin(); mutationIT != mutations.end(); ++mutationIT) {
      for (crossoverIT = crossovers.begin(); crossoverIT != crossovers.end(); ++crossoverIT) {
         Cbin12exp8BeRan->addGenerator(new DEMemberGenerator(*mutationIT, *crossoverIT));
      }
   }

   return Cbin12exp8BeRan;
}

}

// $Log: competitive_member_generator.cpp,v $
// Revision 1.13  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.12  2009/04/30 09:07:42  victor
// + Cbin12exp8BeRan [Radka]
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.11  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.10  2009/01/08 12:23:48  victor
// + Groups of exponential competitive DE algorithms
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.9  2008/12/17 12:51:37  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.8  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.7  2007/11/27 15:32:46  victor
// + DER9 and DEBR18 static members of CompetitiveGenerator
//
// Revision 1.6  2007/11/26 14:22:32  victor
// + Member Generator given a name for identification in reports
//
// Revision 1.5  2007/11/26 12:33:44  victor
// + Member Generator given a name for identification in reports
//
// Revision 1.4  2007/11/21 12:45:37  victor
// + ptrMemberGenerator randomGenerator()
//
// Revision 1.3  2007/11/21 09:06:37  victor
// # Removing dependency on Problem from MemberGenerator constructors
//
// Revision 1.2  2007/11/19 15:14:25  victor
// Added Differential Evolution Algorithm
//
// Revision 1.1  2007/11/19 11:28:59  victor
// Initial revision
//
