/*******************************************************************/ /**
* @file competitive_member_generator.h
*
* @brief Set of competiting generators
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.13 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef COMPETITIVE_MEMBER_GENERATOR_H
#define COMPETITIVE_MEMBER_GENERATOR_H

#include "member_generator.h"
#include "frequency_counter.h"

namespace irafm {

// pouziti navrhoveho vzoru skladba
///Set of competiting generators
class CompetitiveMemberGenerator : public MemberGenerator {
public:
   /// constructor
   CompetitiveMemberGenerator(std::string name): name_(name) {
      /** TODO Volitelne nastaveni parametru delta
       *  urcujiciho hranici degenerace - napr. 1/5 h */
   }

   /// destructor
   virtual ~CompetitiveMemberGenerator() {}

   /// special generators according to DER9 algorithm
   static ptrMemberGenerator DER9();

   /// special generators according to DEBR18 algorithm
   static ptrMemberGenerator DEBR18();

   /// special generators according to Cbin9rl algorithm
   static ptrMemberGenerator Cbin9rl();

   /// special generators according to Cexp9rl algorithm
   static ptrMemberGenerator Cexp9rl(int dimension);

   /// special generators according to Cbin9exp9rl algorithm
   static ptrMemberGenerator Cbin9exp9rl(int dimension);

   /// special generators according to Cbin6exp6rl algorithm
   static ptrMemberGenerator Cbin6exp6rl(int dimension);

   /// special generators according to Cbin12exp8BeRan algorithm
   static ptrMemberGenerator Cbin12exp8BeRan(int dimension);

   virtual Vector generateTrialPoint(ptrProblem problem, Population &population);

   virtual ptrPopulationMember generateNewMember(
      ptrProblem problem, Population &population);

   virtual ptrPopulationMember generateNewMember(
      ptrProblem problem, Population &population, ptrPopulationMember member);

   /// addition of new generator into list of possibilities
   void addGenerator(ptrMemberGenerator generator);

   /// return randomly selected member generator
   ptrMemberGenerator randomGenerator();

   /// feedback - reporting success of the last generated member
   void reportSuccess(const bool success = true);

   virtual std::string getName();

protected:
   /// short name of the generator
   std::string name_;

   /// generators array (selection based on success frequencies)
   std::vector<ptrMemberGenerator> generators_;

   /// success counter for generated members
   FrequencyCounter hitRate_;

   /// index of the last generated member
   index_t lastSelectedGeneratorIndex;

private:
};

}

#endif // COMPETITIVE_MEMBER_GENERATOR_H

// $Log: competitive_member_generator.h,v $
// Revision 1.13  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.12  2009/09/15 13:21:23  victor
// +TODO entries
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.11  2009/04/30 09:07:42  victor
// + Cbin12exp8BeRan [Radka]
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.10  2009/01/08 12:23:48  victor
// + Groups of exponential competitive DE algorithms
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.9  2008/12/17 12:51:37  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.8  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.7  2007/11/27 15:32:46  victor
// + DER9 and DEBR18 static members of CompetitiveGenerator
//
// Revision 1.6  2007/11/26 14:22:32  victor
// + Member Generator given a name for identification in reports
//
// Revision 1.5  2007/11/26 12:33:44  victor
// + Member Generator given a name for identification in reports
//
// Revision 1.4  2007/11/21 12:45:37  victor
// + ptrMemberGenerator randomGenerator()
//
// Revision 1.3  2007/11/21 09:06:37  victor
// # Removing dependency on Problem from MemberGenerator constructors
//
// Revision 1.2  2007/11/19 15:14:25  victor
// Added Differential Evolution Algorithm
//
// Revision 1.1  2007/11/19 11:28:59  victor
// Initial revision
//
