/*******************************************************************/ /**
* @file crossover.cpp
*
* @brief implementation of classes providing crossover
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.6 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

// STL
#include <sstream>

#include "crossover.h"

namespace irafm {

/**********************************************************************
* BinomialCrossover
**********************************************************************/

Vector BinomialCrossover::cross(Vector u, Vector x) {
   size_t dimension = x.size();
   Vector y(dimension);

   math::Interval rndInterval(0, 1);
   index_t crossIndex = math::random(dimension);

   for (index_t j = 0; j < dimension; ++j) {
      if (j == crossIndex) {
         y[j] = u[j];
      }
      else if (rndInterval.randomValue() <= this->CR) {
         y[j] = u[j];
      }
      else {
         y[j] = x[j];
      }
   }

   return y;
}

std::string BinomialCrossover::getName() {
   std::ostringstream result;

   result << "binom[C: " << this->CR << "]";

   return result.str();
}

/**********************************************************************
* ExponentialCrossover
**********************************************************************/

namespace aux {
real P(real CR, real p, size_t d) {
   return pow(CR, d) - d*p*CR + d*p - 1;
}

real dP(real CR, real p, size_t d) {
   return (pow(CR, d - 1) - p)*d;
}
}

real ExponentialCrossover::findCR(real p, size_t dim) {
   using namespace aux;

   real x = 0.5;

   while (abs(P(x, p, dim) > 0.000001)) {
      x = x - P(x, p, dim) / dP(x, p, dim);
   }

   return x;
}

Vector ExponentialCrossover::cross(Vector u, Vector x) {
   size_t dimension = x.size();
   Vector y(dimension);

   math::Interval rndInterval(0, 1);

   // first index, from which mutation begins
   index_t crossIndex = math::random(dimension);

   for (index_t j = 0; j < dimension; ++j) {
      index_t i = (j + crossIndex) % dimension;
      y[i] = u[i];

      if (rndInterval.randomValue() > this->CR) {
         break;
      }
   }

   return y;
}

std::string ExponentialCrossover::getName() {
   std::ostringstream result;

   result << "exp[C: " << this->CR << "]";

   return result.str();
}

}

// $Log: crossover.cpp,v $
// Revision 1.6  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2009/01/08 12:23:48  victor
// + Groups of exponential competitive DE algorithms
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.3  2008/12/17 18:53:08  victor
// Ensuring at least one change in exp crossover
//
// Revision 1.2  2008/12/17 13:46:53  victor
// + Exponential crossover
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.1  2008/12/17 12:51:37  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
