/*******************************************************************/ /**
* @file de_member_generator.cpp
*
* @brief Abstract new member generator according to DE Algorithm
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.4 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

// STL
#include <sstream>

#include "de_member_generator.h"

namespace irafm {

Vector DEMemberGenerator::generateDEPoint(
   ptrPopulationMember x,
   Population &population
) {
   // temporarilly remove x from population to ensure that will not be
   // chosen as another random member
   population.removeMember(x);

   Vector u = mutator->generateMutant(population);

   // return temporarilly removed x to its previous place
   population.addMember(x);

   return crossover->cross(u, x->data);
}

Vector DEMemberGenerator::generateTrialPoint(
   ptrProblem problem, Population &population
) {
   ptrPopulationMember x = population.randomSelect();

   return generateDEPoint(x, population);
}

ptrPopulationMember DEMemberGenerator::generateNewMember(
   ptrProblem problem, Population &population, ptrPopulationMember member) {

   Vector dePoint(problem->dimension());

   size_t iteration = 0;

   do {
      dePoint = generateDEPoint(member, population);
      ++iteration;
   }
   while ((!problem->isInSearchSpace(dePoint)) && (iteration < 300));

   /// @todo Unify constants for internal iteration limit
   if (!(iteration < 300)) {
      dePoint = problem->boundToSearchSpace(dePoint);
   }

   return problem->newMember(dePoint);
}

std::string DEMemberGenerator::getName() {
   std::ostringstream result;

   result
   << "DE gen["
   << "mutation: " << this->mutator->getName() << ", "
   << "cross: " << this->crossover->getName()
   << "]";

   return result.str();
}

}

// $Log: de_member_generator.cpp,v $
// Revision 1.4  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.3  2009/04/21 12:19:56  victor
// # todo management
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.2  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.1  2008/12/17 12:51:37  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
