// STL
#include <iostream>
#include <fstream>

// IRAFM
#include <irafm/ea.h>

using namespace std;
using namespace irafm;

const int dim = 5;

template <typename BenchmarkFunction>
class BaseProblem: public Problem {
public:
   class Member : public PopulationMember {
   public:
      Member(Vector aData): PopulationMember(aData) {}

      virtual real criteria() const {
         return BenchmarkFunction()(data);
      }
   };

   BaseProblem(std::string name): Problem(name) {}

   virtual ptrPopulationMember newMember(const Vector aData) const {
      return new Member(aData);
   }

   virtual Population initialPopulation() const {
      return randomPopulation(10 * dimension());
   }
};

class Dejong1Problem: public BaseProblem<DeJong1Function> {
public:
   Dejong1Problem(): BaseProblem<DeJong1Function>("Dejong1 function") {
      for (int i = 0; i < dim; ++i) {
         searchSpace_.addDimension(math::Interval(-5.12, 5.12));
      }
   }
};

class RastriginProblem: public BaseProblem<RastriginFunction> {
public:
   RastriginProblem(): BaseProblem<RastriginFunction>("Rastrigin function") {
      for (int i = 0; i < dim; ++i) {
         searchSpace_.addDimension(math::Interval(-5.12, 5.12));
      }
   }
};

class SchwefelProblem: public BaseProblem<SchwefelFunction> {
public:
   SchwefelProblem(): BaseProblem<SchwefelFunction>("Schwefel function") {
      for (int i = 0; i < dim; ++i) {
         searchSpace_.addDimension(math::Interval(-500, 500));
      }
   }
};

class AckleyProblem: public BaseProblem<AckleyFunction> {
public:
   AckleyProblem(): BaseProblem<AckleyFunction>("Ackley function") {
      for (int i = 0; i < dim; ++i) {
         searchSpace_.addDimension(math::Interval(-30, 30));
      }
   }
};

class RosenProblem: public BaseProblem<RosenFunction> {
public:
   RosenProblem(): BaseProblem<RosenFunction>("Rosen function") {
      for (int i = 0; i < dim; ++i) {
         searchSpace_.addDimension(math::Interval(-2048, 2048));
      }
   }
};

class GriewankProblem: public BaseProblem<GriewankFunction> {
public:
   GriewankProblem(): BaseProblem<GriewankFunction>("Griewank function") {
      for (int i = 0; i < dim; ++i) {
         searchSpace_.addDimension(math::Interval(-400, 400));
      }
   }
};

int main() {
   ProblemSuite problemSuite(100);
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::DER9()));
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::DEBR18()));
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::Cbin9rl()));
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::Cexp9rl(dim)));
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::Cbin9exp9rl(dim)));
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::Cbin6exp6rl(dim)));
   problemSuite.addAlgorithm(new DEAlgorithm(CompetitiveMemberGenerator::Cbin12exp8BeRan(dim)));

   problemSuite.addProblem(new AckleyProblem());
   problemSuite.addProblem(new Dejong1Problem());
   problemSuite.addProblem(new GriewankProblem());
   problemSuite.addProblem(new RastriginProblem());
   problemSuite.addProblem(new RosenProblem());
   problemSuite.addProblem(new SchwefelProblem());

   ofstream results("results.txt");

   ptrReporter reporter(
      new DualReporter(
         new StreamReporter(cout, rlFull),
         new StreamReporter(results, rlResults)
      )
   );

   cout << std::endl;

   problemSuite.run(reporter);

   cout << std::endl;

   results.close();

   return 0;
}

// $Log: main.cpp,v $
// Revision 1.11  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.10  2009/04/30 09:08:08  victor
// + Griewang, Rosen, Cbin12exp8BeRan [Radka]
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.9  2009/03/26 11:16:07  victor
// + Makefile
// - imp removed
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.8  2009/01/08 12:23:48  victor
// + Groups of exponential competitive DE algorithms
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.7  2008/12/17 14:06:47  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2008/12/11 10:21:16  victor
// Definition and usage of report levels
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2008/01/30 12:42:08  victor
// Documenting the source code (main page)
//
// Revision 1.3  2007/12/13 15:14:38  victor
// + Ackley problem
// # Report parametrized by Reporter policy
//
// Revision 1.2  2007/11/27 15:32:47  victor
// + DER9 and DEBR18 static members of CompetitiveGenerator
//
// Revision 1.1  2007/11/26 14:25:47  victor
// Initial revision
//
