/*******************************************************************/ /**
* @file frequency_counter.cpp
*
* @brief Manipulation for frequency counters
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.7 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#include "frequency_counter.h"

namespace irafm {

void FrequencyCounter::setNumItems(size_t numItems) {
   numItems_ = numItems;

   reset();
}

FrequencyCounter::FrequencyCounter(size_t numItems, double delta): delta_(delta) {
   setNumItems(numItems);
}

void FrequencyCounter::reset(frequency_t freqValue) {
   freqArray_.clear();

   for (size_t i = 0; i < numItems_; ++i) {
      freqArray_.push_back(freqValue);
   }

   freqSum_ = numItems_ * freqValue;
}

void FrequencyCounter::inc(index_t freqIndex) {
   ++frequency(freqIndex);
   ++freqSum_;

   repairDegeneration();
}

size_t& FrequencyCounter::frequency(index_t freqIndex) {
   return freqArray_[freqIndex];
}

real FrequencyCounter::probability(index_t freqIndex) {
   return real(frequency(freqIndex)) / real(freqSum_);
}

bool FrequencyCounter::checkDegeneration() {
   for (index_t freqIndex = 0; freqIndex < numItems_; ++freqIndex) {
      if (probability(freqIndex) < delta_) {
         return true;
      }
   }

   return false;
}

void FrequencyCounter::repairDegeneration() {
   /** @todo Inform user about degeneration when rlInternals report level is set */
   if (checkDegeneration())
      reset();
}

index_t FrequencyCounter::randomItemIndex() {
   size_t randomValue = rand() % freqSum_;

   size_t partialSum = 0;

   for (index_t freqIndex = 0; freqIndex < numItems_; ++freqIndex) {
      partialSum = partialSum + frequency(freqIndex);

      if (randomValue < partialSum) {
         return freqIndex;
      }
   }

   return numItems_;
}

}

// $Log: frequency_counter.cpp,v $
// Revision 1.7  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2009/04/21 12:21:31  victor
// # degeneration checking
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.3  2007/12/04 15:05:18  victor
// * Resolving error in FTransform Tuning app
//
// Revision 1.2  2007/11/19 11:29:39  victor
// Added Competitive Member Generator
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
