/*******************************************************************/ /**
* @file frequency_counter.h
*
* @brief Manipulation for frequency counters
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.6 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef FREQUENCY_COUNTER_H
#define FREQUENCY_COUNTER_H

// STL
#include <vector>

//IRAFM
#include <irafm/math.h>

namespace irafm {

/// Manipulation for frequency counters
class FrequencyCounter {
public:
   /// type holding number of occurences
   typedef size_t frequency_t;

   /// implicit constructor
   FrequencyCounter() {}

   /// constructor
   FrequencyCounter(size_t numItems, double delta = 0.001);

   /// destructor
   virtual ~FrequencyCounter() {}

   /// set number of items to monitor
   void setNumItems(size_t numItems);

   /// reset all frequencies to the same value
   void reset(frequency_t freqValue = 2);

   /// increment frequency of given item index
   void inc(index_t freqIndex);

   /// get frequency of given item index
   size_t& frequency(index_t freqIndex);

   /// get probability of given item index
   real probability(index_t freqIndex);

   /// checks whether some probability decreased under limit value
   bool checkDegeneration();

   /// repairs posible degeneration
   void repairDegeneration();

   /// generate random item index according to current probabilities
   index_t randomItemIndex();

protected:
   /// number of items to monitor
   size_t numItems_;

   /// sum of all occurences
   frequency_t freqSum_;

   double delta_;

   /// holder of frequencies
   std::vector<frequency_t> freqArray_;

private:
};

}

#endif // FREQUENCY_COUNTER_H

// $Log: frequency_counter.h,v $
// Revision 1.6  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2009/04/21 12:21:31  victor
// # degeneration checking
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.3  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.2  2007/11/19 11:29:39  victor
// Added Competitive Member Generator
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
