/*******************************************************************/ /**
* @file population.cpp
*
* @brief Implementation of the Population manipulations
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.9 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

// STL
#include <stdexcept>
#include <cstdlib>
#include <sstream>

#include "population.h"

//#include "UnitMain.h"

namespace irafm {

using namespace std;

string Population::printPopulation() {
   std::ostringstream result;

   MembersArray::iterator it;
   ptrPopulationMember member;

   for (it = members_.begin(); it != members_.end(); ++it) {
      member = *it;
      result << member->print() << "  ";
   }

   return result.str();
}

void Population::addMember(const ptrPopulationMember aMember) {
   MembersArray::iterator it;
   ptrPopulationMember member;

   for (it = members_.begin(); it != members_.end(); ++it) {
      member = *it;

      if (aMember->objectiveFunction() < member->objectiveFunction()) {
         break;
      }
   }

   if (it == members_.end()) {
      members_.push_back(aMember);
   }
   else {
      members_.insert(it, aMember);
   }

}

void Population::addMembers(Population &members) {
   MembersArray::iterator it;
   ptrPopulationMember member;

   for (it = members.members_.begin(); it != members.members_.end(); ++it) {
      member = *it;
      addMember(member);
   }
}

void Population::checkRange(const index_t index) {
   if (size() < 1)
      throw std::out_of_range("No members in population");

   if ((index >= size()) || (index < 0))
      throw std::out_of_range("Population member index out of range");
}

ptrPopulationMember Population::getMember(
   const index_t index, const bool flRemove
) {
   checkRange(index);

   ptrPopulationMember result = members_[index];

   if (flRemove == rsRemove) removeMember(index);

   return result;
}

ptrPopulationMember Population::best(const bool flRemove) {
   return getMember(0, flRemove);
}

ptrPopulationMember Population::worst(const bool flRemove) {
   return getMember(size() - 1, flRemove);
}

bool Population::replaceWorst(ptrPopulationMember member) {
   if (worst() < member) return false;

   removeMember(size() - 1);
   addMember(member);

   return true;
}

void Population::removeMember(const index_t index) {
   ptrPopulationMember memberToRemove = getMember(index, rsLeave);
   removeMember(memberToRemove);
}

void Population::removeMember(ptrPopulationMember memberToRemove) {
   Population::MembersArray::iterator it;

   for (it = members_.begin(); it != members_.end(); ++it) {
      if (memberToRemove == *it) {
         members_.erase(it);

         return;
      }
   }
}

index_t Population::randomIndex() {
   if (size() < 1)
      throw std::out_of_range("No members in population");

   return rand() % size();
}

/****************************************************************/ /**
* random selection of member
* @param flRemove flag determining whether selected member will be
*        removed from population or will stay there. Possible values
*        are: rsLeave, rsRemove
*/
ptrPopulationMember Population::randomSelect(const bool flRemove) {
   index_t rndIndex = randomIndex();

   ptrPopulationMember result = getMember(rndIndex, flRemove);

   return result;
}

Population Population::randomSelect(const size_t count, const bool flRemove) {
   Population selection;

   // nahodny vyber a odebrani daneho poctu clenu z populace
   for (size_t i = 0; i < count; ++i) {
      selection.addMember(randomSelect(rsRemove));
   }

   // vraceni odebranych prvku, pouze v pripade, ze se mely ponechat
   if (flRemove == rsLeave) {
      addMembers(selection);
   }

   return selection;
}

Vector centroid(Population &population) {
   Vector g = population.getMember(0)->data;

   for (index_t i = 1; i < population.size(); ++i) {
      g = g + population.getMember(i)->data;
   }

   return g / real(population.size());
}

}

// $Log: population.cpp,v $
// Revision 1.9  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.8  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.7  2008/12/17 12:51:37  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.5  2007/12/04 15:05:18  victor
// * Resolving error in FTransform Tuning app
//
// Revision 1.4  2007/11/21 09:05:28  victor
// + Counting evaluation of objective function
// + Caching value of objective function
//
// Revision 1.3  2007/11/15 15:40:45  victor
// Simple refactoring
//
// Revision 1.2  2007/11/15 08:52:15  victor
// Implementing centroid (removed from SimplexGenerator) for usage in other places
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
