/*******************************************************************/ /**
* @file population.h
*
* @brief Container for population members
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.7 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef POPULATION_H
#define POPULATION_H

// STL
#include <set>

#include "population_member.h"

#define rsRemove true
#define rsLeave false

namespace irafm {

/// Container for population members
class Population {
public:
   /// prints entire population to std::string
   std::string printPopulation();

   /// add member to current population
   void addMember(const ptrPopulationMember aMember);

   /// add all members from another population
   void addMembers(Population &members);

   /// remove member of given index from population
   void removeMember(const index_t index);

   /// remove member from population
   void removeMember(ptrPopulationMember memberToRemove);

   /// return member of population of given index
   ptrPopulationMember getMember(const index_t index,
                                 const bool flRemove = rsLeave);

   /// return best member acording to member criteria
   ptrPopulationMember best(const bool flRemove = rsLeave);

   /// return worst member acording to member criteria
   ptrPopulationMember worst(const bool flRemove = rsLeave);

   /// replace worst member in population by given member,
   /// but only if given member is better
   bool replaceWorst(ptrPopulationMember member);

   /// generate random member index of current population
   index_t randomIndex();

   /// random selection of member
   ptrPopulationMember randomSelect(const bool flRemove = rsLeave);

   /// random selection of several members
   Population randomSelect(const size_t count, const bool flRemove = rsLeave);

   /// number of all members in population
   size_t size() {
      return members_.size();
   }
private:
   /// internal storage of members
   typedef std::vector<ptrPopulationMember> MembersArray;

   /// container of all members in population
   MembersArray members_;

   /// checks whether index is not out of boundaries
   void checkRange(const index_t index);
};

/// smart pointer to Population
typedef yasper::ptr<Population> ptrPopulation;

/// computes centroid of given population
Vector centroid(Population &population);

}

#endif // POPULATION_H

// $Log: population.h,v $
// Revision 1.7  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
// Revision 1.4  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.3  2007/11/15 15:40:45  victor
// Simple refactoring
//
// Revision 1.2  2007/11/15 08:52:15  victor
// Implementing centroid (removed from SimplexGenerator) for usage in other places
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
