/*******************************************************************/ /**
* @file population_member.h
*
* @brief Contains definition of base structural element
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.7 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef POPULATION_MEMBER_H
#define POPULATION_MEMBER_H

// STL
#include <string>

// IRAFM
#include <irafm/math.h>
#include <irafm/smart_ptr.h>

namespace irafm {

/// Essential structural element of population
class PopulationMember {
public:
   /// internal storage for member data
   Vector data;

   /// implicit constructor
   PopulationMember(): flCached(false) {}

   /// initializing constructor
   PopulationMember(Vector aData): data(aData), flCached(false) {}

   /// destructor
   virtual ~PopulationMember() {}

   /// objective function which should be minimalized
   real objectiveFunction();

   /// textual representation used in reports
   virtual std::string print();

   /// dimension of member encoded to vector space
   virtual size_t dimension() const {
      return data.size();
   }

private:
   /// flag that value is already cached
   bool flCached;

   /// casched value of objective function
   real cachedValue;

   /// crucial block of code to be implemented in inherited class by user
   virtual real criteria() const abstract;

public:
   /// static members for evaluation counting
   static void resetCounter();

   static size_t numberOfCalls();

private:
   static size_t numCalls_;
};

/// smart pointer to PopulationMember
typedef yasper::ptr<PopulationMember> ptrPopulationMember;

/// comparison operators of population members
bool operator<(ptrPopulationMember m1, ptrPopulationMember m2);
bool operator<=(ptrPopulationMember m1, ptrPopulationMember m2);

}

#endif // POPULATION_MEMBER_H

// $Log: population_member.h,v $
// Revision 1.7  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2008/12/17 12:51:37  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2008/04/07 09:12:17  victor
// Managing TODO items
//
// Revision 1.4  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.3  2007/12/04 15:05:18  victor
// * Resolving error in FTransform Tuning app
//
// Revision 1.2  2007/11/21 09:05:28  victor
// + Counting evaluation of objective function
// + Caching value of objective function
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
