/*******************************************************************/ /**
* @file problem.h
*
* @brief Abstract class of task settings
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.7 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef PROBLEM_H
#define PROBLEM_H

/*******************************************************************/ /**
@mainpage The Stochastic Algorithms for Global Optimization C++ library

The problem of the global optimization is hard and plenty of stochastic
algorithms were proposed for its solution. The authors of many such
stochastic algorithms claim the efficiency and the reliability of
searching for the global minimum. The reliability means that the point
with minimal function value found in the search process is sufficiently
close to the global minimum point and the efficiency means that the
algorithm finds a point sufficiently close to the global minimum point
at reasonable time. However, when we use such algorithms, we face the
problem of the setting their control parameters.

The efficiency and the reliability of many algorithms are strongly
dependent on the values of control parameters. Recommendations given
by authors are often vague or uncertain. A user is supposed to be able
to change the parameter values according to the results of trial-and-error
preliminary experiments with the search process. Such attempt is not
acceptable in tasks, where the global optimization is one step on the
way to the solution of the user's problem or when the user has no
experience in fine art of control parameter tuning. Adaptive robust
algorithms reliable enough at reasonable time-consumption without the
necessity of fine tuning their input parameters have been studied in
recent years.

The differential evolution (DE) has become one of the most popular
algorithms for the continuous global optimization problems in last
decade years. But it is known that the efficiency of the search for
the global minimum is very sensitive to the setting of its control
parameters. That is why the self-adaptive DE was proposed and included
into this library.

This C++ library brings essential classes, data types and algorithms
for usage of the proposed differential evolution algorithms. It is licensed
as a GNU GPL 3 software library providing code for effective construction
of user's own programs using DE optimization. The library is provided as
a package of header and implementation files (*.h and *.cpp) together
hyper textual documentation and demonstrational examples of usage
(the user is encouraged to consult them first). The user's resulting
project integrating our library and user's own code could be compiled
by GNU GCC compiler (version 3.4.5).

Simple example of usage:
@include "DEComp\main.cpp"

************************************************************************/

// STL
#include <string>

// IRAFM
#include <irafm/math.h>

#include "population.h"

namespace irafm {

/// Abstract class of task settings
class Problem {
public:
   /// name of the problem
   std::string name;

   /// implicit constructor
   Problem(std::string aName): name(aName) {}

   /// constructor
   Problem(std::string aName, math::DomainBox &searchSpace):
         name(aName), searchSpace_(searchSpace) {}

   virtual ~Problem() {}

   /// dimension of the problem
   virtual size_t dimension() const {
      return searchSpace_.dimension();
   }

   /// create new member
   virtual ptrPopulationMember newMember(const Vector aData) const abstract;

   /// get new random member
   virtual ptrPopulationMember newRandomMember() const;

   /// create new random population of size count
   virtual Population randomPopulation(const int count) const;

   /// generate initial population
   virtual Population initialPopulation() const abstract;

   /// checks whether given vector is in box constrains
   bool isInSearchSpace(const Vector v) const {
      return searchSpace_.containsVector(v);
   }

   /// trims vector coordinates to fit in search space
   Vector boundToSearchSpace(const Vector v) const {
      return searchSpace_.bound(v);
   }

protected:
   /// box constraint data structure
   math::DomainBox searchSpace_;
};

/// smart pointer to Problem
typedef yasper::ptr<Problem> ptrProblem;

}

#endif // PROBLEM_H

// $Log: problem.h,v $
// Revision 1.7  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2008/01/30 12:42:02  victor
// Documenting the source code (main page)
//
// Revision 1.5  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.4  2007/12/04 15:01:48  victor
// + Vector boundToSearchSpace(const Vector v) const
//
// Revision 1.3  2007/11/26 09:36:27  victor
// + Problem given a name for identification in reports
//
// Revision 1.2  2007/11/15 15:40:45  victor
// Simple refactoring
//
// Revision 1.1  2007/11/14 13:37:18  victor
// Initial revision of EA lib
//
