/*******************************************************************/ /**
* @file problem_suite.cpp
*
* @brief Classes for batch processing and report generation
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.9 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

// STL
#include <sstream>
#include <iostream>

#include <math.h>

#include "problem_suite.h"

namespace irafm {

using namespace std;

real ProblemStudy::stdDevEvaluations() {
   real avg = real(nEvaluations) / nExperiments_;

   real sumSquares = 0;
   std::vector<int>::iterator it;

   for (it = arrEvaluations.begin(); it < arrEvaluations.end(); ++it) {
      int evals = *it;
      real difference = evals - avg;
      sumSquares += difference * difference;
   }

   return sqrt(sumSquares / (arrEvaluations.size() - 1));
}

real ProblemStudy::stdDevSuccEvaluations() {
   real avgSucc = real(nSuccEvaluations) / nSuccesses;

   real sumSquares = 0;
   std::vector<int>::iterator it;

   for (it = arrSuccEvaluations.begin(); it < arrSuccEvaluations.end(); ++it) {
      int evals = *it;
      real difference = evals - avgSucc;
      sumSquares += difference * difference;
   }

   return sqrt(sumSquares / (arrSuccEvaluations.size() - 1));
}

void ProblemStudy::run(ptrReporter reporter) {
   nSuccesses = 0;
   nGenerations = 0;
   nSuccGenerations = 0;
   nEvaluations = 0;
   nSuccEvaluations = 0;
   bestPrecission = 10000;

   reporter->beginStudy(this);

   ptrPopulationMember best;

   for (size_t expNo = 0; expNo < nExperiments_; ++expNo) {
      reporter->beginExperiment();

      best = algorithm_->optimize(problem_, reporter);
      real precission = best->objectiveFunction();

      if (precission < bestPrecission) {
         bestPrecission = precission;
      }

      nGenerations += algorithm_->getNumberOfIterations();
      nEvaluations += PopulationMember::numberOfCalls();
      arrEvaluations.push_back(PopulationMember::numberOfCalls());

      if (algorithm_->getStatus() == fsSuccess) {
         ++nSuccesses;
         nSuccGenerations += algorithm_->getNumberOfIterations();
         nSuccEvaluations += PopulationMember::numberOfCalls();
         arrSuccEvaluations.push_back(PopulationMember::numberOfCalls());
      }
      else {
         failurePrecisionSum += precission;
      }
   }

   reporter->endStudy(this);
}

void ProblemSuite::addProblem(ptrProblem problem) {
   problems_.push_back(problem);
}

void ProblemSuite::addAlgorithm(ptrAlgorithm algorithm) {
   algorithms_.push_back(algorithm);
}

void ProblemSuite::run(ptrReporter reporter) {
   std::vector<ptrAlgorithm>::iterator algorithmIT;
   std::vector<ptrProblem>::iterator problemIT;

   for (algorithmIT = algorithms_.begin(); algorithmIT != algorithms_.end(); ++algorithmIT) {
      ptrAlgorithm algorithm_ptr = *algorithmIT;

      for (problemIT = problems_.begin(); problemIT != problems_.end(); ++problemIT) {
         ptrProblem problem_ptr = *problemIT;

         ProblemStudy study(algorithm_ptr, problem_ptr, nRepeats);
         study.run(reporter);
      }
   }
}

}

// $Log: problem_suite.cpp,v $
// Revision 1.9  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.8  2009/05/04 08:44:47  victor
// + Vypocet a vypis smerodatnych odchylek poctu vyhodnoceni obj. funkce
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.7  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
// Revision 1.4  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.3  2007/12/13 15:15:49  victor
// # Report parametrized by Reporter policy
//
// Revision 1.2  2007/11/26 14:24:23  victor
// + Added empty line after each problem report
//
// Revision 1.1  2007/11/26 12:16:03  victor
// Initial revision
//
