/*******************************************************************/ /**
* @file problem_suite.h
*
* @brief Classes for batch processing and report generation
*
* @author Viktor PAVLISKA, Hashim HABIBALLA, Jiri BLAZEK
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.8 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef PROBLEM_SUITE_H
#define PROBLEM_SUITE_H

// STL
#include <vector>
#include <string>

namespace irafm {
class ProblemStudy;
}

#include "algorithm.h"
#include "reporter.h"

namespace irafm {

/// Problemova studie sestavajici z nekolikanasobneho volani optimalizace
class ProblemStudy {
public:
   ProblemStudy(ptrAlgorithm algorithm, ptrProblem problem,
                size_t numberOfRepeats = 100):
         algorithm_(algorithm),
         problem_(problem),
         nExperiments_(numberOfRepeats) {}

   /// starts problem study
   void run(ptrReporter reporter);

   real successRate() {
      return real(100 * nSuccesses) / nExperiments_;
   }

   size_t nFailures() {
      return nExperiments_ - nSuccesses;
   }

   size_t avgGenerations() {
      return nGenerations / nExperiments_;
   }

   size_t avgSuccGenerations() {
      return nSuccGenerations / nSuccesses;
   }

   size_t avgEvaluations() {
      return nEvaluations / nExperiments_;
   }

   real stdDevEvaluations();

   size_t avgSuccEvaluations() {
      return nSuccEvaluations / nSuccesses;
   }

   real stdDevSuccEvaluations();

   real avgFailurePrecision() {
      return failurePrecisionSum / nFailures();
   }

   /// number of succesful optimalization runs
   size_t nSuccesses;

   /// number of generations
   size_t nGenerations;

   /// number of generations in succesful experiments
   size_t nSuccGenerations;

   /// number of objective function evaluations
   size_t nEvaluations;

   /// array of numbers of objective function evaluations
   std::vector<int> arrEvaluations;

   /// number of objective function evaluations in succesful experiments
   size_t nSuccEvaluations;

   /// array of numbers of objective function evals in succesful experiments
   std::vector<int> arrSuccEvaluations;

   /// best achieved precission
   real bestPrecission;

private:
   friend class StreamReporter;

   /// algoritmus optimalizace
   ptrAlgorithm algorithm_;

   /// problem pro optimalizaci
   ptrProblem problem_;

   /// number of problem instance tries to optimize
   const size_t nExperiments_;

   /// sum of average precision in unsuccesful experiments
   real failurePrecisionSum;

};

/// Testing of algorithms vs. problems
class ProblemSuite {
public:
   /// constructor
   ProblemSuite(size_t numOfRepeats = 100): nRepeats(numOfRepeats) {};

   /// destructor
   ~ProblemSuite() {};

   /// adds problem into problem set
   void addProblem(ptrProblem problem);

   /// adds algorithm into algorithm set
   void addAlgorithm(ptrAlgorithm algorithm);

   /// starts problem suite
   void run(ptrReporter reporter);

protected:
   /// internal problems storage
   std::vector<ptrProblem> problems_;

   /// internal algortihms storage
   std::vector<ptrAlgorithm> algorithms_;

   /// number of experiments count in each study
   size_t nRepeats;

private:
};

}

#endif // PROBLEM_SUITE_H

// $Log: problem_suite.h,v $
// Revision 1.8  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.7  2009/05/04 08:44:47  victor
// + Vypocet a vypis smerodatnych odchylek poctu vyhodnoceni obj. funkce
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
// Revision 1.3  2008/01/28 11:04:05  victor
// Documenting the source code
//
// Revision 1.2  2007/12/13 15:15:49  victor
// # Report parametrized by Reporter policy
//
// Revision 1.1  2007/11/26 12:16:03  victor
// Initial revision
//
