/*******************************************************************/ /**
* @file reporter.cpp
*
* @brief Implementation of report classes
*
* @author Viktor PAVLISKA, Hashim HABIBALLA
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.8 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

// STL
#include <iostream>
#include <sstream>

#include "reporter.h"

namespace irafm {

using namespace std;

void StreamReporter::print(std::string str) {
   if (reportLevel == rlQuiet) return;

   out << str;
}

void StreamReporter::beginStudy(ProblemStudy *problemStudy) {
   experimentIndex = 0;

   if (reportLevel & rlInfo) {
      out
      << "Starting new study: " << endl
      << "Method: " << problemStudy->algorithm_->getName() << endl
      << "Task: " << problemStudy->problem_->name << endl;
   }
}

void StreamReporter::endStudy(ProblemStudy *problemStudy) {
   if (reportLevel & rlResults) {
      out
      << endl << endl
      << "Optimalization study report:" << endl
      << "----------------------------" << endl
      << "Method: " << problemStudy->algorithm_->getName() << endl
      << "Task: " << problemStudy->problem_->name << endl
      << "Number of experiments: " << problemStudy->nExperiments_ << endl
      << "Success rate: " << problemStudy->successRate() << "%" << endl
      << "Avg number of generations: " << problemStudy->avgGenerations() << endl
      << "Avg number of obj. function evaluations: " << problemStudy->avgEvaluations() << endl
      << "Standard deviation of obj. function evaluations: "
      << problemStudy->stdDevEvaluations() << endl
      << "Best achieved precision: " << problemStudy->bestPrecission << endl;

      /// TODO Logaritmicke mereni presnosti: log(|c-m| / |c|)

      /// TODO Pridat vypis souradnic nalezeneho optima

      /// TODO Do vystupu zaradit dimenzi reseneho problemu

      if ((problemStudy->nSuccesses > 0) && (problemStudy->nSuccesses != problemStudy->nExperiments_)) {
         out
         << "Avg number of generations in successful experiments: "
         << problemStudy->avgSuccGenerations() << endl
         << "Avg number of obj. function evaluations in successful experiments: "
         << problemStudy->avgSuccEvaluations() << endl
         << "Standard deviation of obj. function evaluations in successful experiments: "
         << problemStudy->stdDevSuccEvaluations() << endl;
      }

      if (problemStudy->nFailures() > 0) {
         out
         << "Avg precision in unsuccessful experiments: "
         << problemStudy->avgFailurePrecision() << endl;
      }

      /** TODO Print information about frequencies of competiting heuristics
       *  when report level is set to rlInternals
       */

      out << endl;
   }
}

void StreamReporter::beginExperiment() {
   iterationIndex = 0;
   ++experimentIndex;

   if (reportLevel & rlProgress) {
      out << endl << "exp #" << experimentIndex << " ";
   }
}

void StreamReporter::endExperiment() {
}

void StreamReporter::beginIteration(Algorithm *algorithm) {
   if (iterationIndex++ % 10 == 0) {
      if (reportLevel & rlProgress) {
         out << ".";
      }
   }
}

void StreamReporter::endIteration(Algorithm *algorithm) {
}

}

// $Log: reporter.cpp,v $
// Revision 1.8  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.7  2009/09/15 13:21:23  victor
// +TODO entries
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.6  2009/05/04 08:43:25  victor
// # Zmena ukoncovaci podminky
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2009/04/21 12:20:38  victor
// + internal report level
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.3  2008/12/11 10:21:16  victor
// Definition and usage of report levels
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.2  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.1  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
