/*******************************************************************/ /**
* @file reporter.h
*
* @brief Classes for reports of optimalization
*
* @author Viktor PAVLISKA, Hashim HABIBALLA
*
* @par Copyright  2007 IRAFM
*   Institute for Research and Application of Fuzzy Modeling           @n
*   University of Ostrava                                              @n
*   30. dubna 22                                                       @n
*   701 03, Ostrava 1                                                  @n
*   Czech Republic                                                     @n
*
*   This file is part of The Stochastic Algorithms for Global
*   Optimization C++ library.
*
*   This library is free software which you can redistribute and/or
*   modify under the terms of the GNU Library General Public License
*   version 3.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty
*   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*   See the GNU General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library.  If not, write to the Free Software
*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Revision: 1.6 $
* $Date: 2010/03/23 10:07:49 $
************************************************************************/

#ifndef REPORTER_H
#define REPORTER_H

#include <string>
#include <fstream>

#include <irafm/smart_ptr.h>

namespace irafm {

class Reporter;

/// smart pointer to Algorithm
typedef yasper::ptr<Reporter> ptrReporter;

}

#include "population.h"
#include "problem_suite.h"
#include "algorithm.h"

namespace irafm {

/// Determines level of report information
enum ReportLevel {
   rlQuiet = 0,                    ///<: no information is written to output streams
   rlResults = 1,                  ///<: reports results of computation
   rlInfo = 2,                     ///<: reports information about current state
   rlProgress = 4,                 ///<: reports progress of computation
   rlMedium = rlInfo | rlResults,  ///<: combination of results and informations
   rlFull = rlMedium | rlProgress, ///<: full report
   rlInternals = 8                 ///<: internal messages generated during optimalization
};

class Reporter {
public:
   Reporter() {};

   virtual ~Reporter() {};

   /// print message to output
   virtual void print(std::string str) abstract;

   /// report start of a problem study
   virtual void beginStudy(ProblemStudy *problemStudy) abstract;

   /// report end of a problem study
   virtual void endStudy(ProblemStudy *problemStudy) abstract;

   /// report start of iteration (new generation)
   virtual void beginIteration(Algorithm *algorithm) abstract;

   /// report end of iteration (new generation)
   virtual void endIteration(Algorithm *algorithm) abstract;

   /// report start of an experiment
   virtual void beginExperiment() abstract;

   /// report end of an experiment
   virtual void endExperiment() abstract;

protected:

private:
};

class StreamReporter: public Reporter {
public:
   StreamReporter(std::ostream &output_stream, ReportLevel rl = rlMedium) :
         out(output_stream), reportLevel(rl) {};

   virtual ~StreamReporter() {};

   virtual void print(std::string str);

   virtual void beginStudy(ProblemStudy *problemStudy);
   virtual void endStudy(ProblemStudy *problemStudy);

   virtual void beginIteration(Algorithm *algorithm);
   virtual void endIteration(Algorithm *algorithm);

   virtual void beginExperiment();
   virtual void endExperiment();

protected:
   std::ostream &out; ///< output stream

   ReportLevel reportLevel; ///< level of report details
   int iterationIndex;
   int experimentIndex;

private:
};

class DualReporter: public Reporter {
public:
   DualReporter(ptrReporter r1, ptrReporter r2): rep1(r1), rep2(r2) {}

   virtual void print(std::string str) {
      rep1->print(str);
      rep2->print(str);
   }

   virtual void beginStudy(ProblemStudy *problemStudy) {
      rep1->beginStudy(problemStudy);
      rep2->beginStudy(problemStudy);
   }

   virtual void endStudy(ProblemStudy *problemStudy) {
      rep1->endStudy(problemStudy);
      rep2->endStudy(problemStudy);
   }

   virtual void beginIteration(Algorithm *algorithm) {
      rep1->beginIteration(algorithm);
      rep2->beginIteration(algorithm);
   }

   virtual void endIteration(Algorithm *algorithm) {
      rep1->endIteration(algorithm);
      rep2->endIteration(algorithm);
   }

   virtual void beginExperiment() {
      rep1->beginExperiment();
      rep2->beginExperiment();
   }

   virtual void endExperiment() {
      rep1->endExperiment();
      rep2->endExperiment();
   }

protected:
   ptrReporter rep1, rep2;
};

}

#endif // REPORTER_H

// $Log: reporter.h,v $
// Revision 1.6  2010/03/23 10:07:49  victor
// removing namespace
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.5  2009/04/21 12:20:38  victor
// + internal report level
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.4  2008/12/17 14:06:47  victor
// # Changed structure of DE member generator
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.3  2008/12/11 10:21:16  victor
// Definition and usage of report levels
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.2  2008/12/10 11:49:02  victor
// Parametrizing the ProblemSuite
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.1  2008/11/23 15:06:30  victor
// Working on ProblemStudy and ProblemReporter
//
