#ifndef interval_h
#define interval_h

// STL
#include <algorithm>
#include <iostream>

#include "real.h"
#include "functions.h"

namespace irafm {

namespace math {

class Interval {
public:
   real left;  //< left bound of interval
   real right; //< right bound of interval

   /// default constructor creates empty interval
   Interval(): left(1), right(-1) {}

   /// initializing constructor
   Interval(real _left, real _right): left(_left), right(_right) {}

   /// width of interval
   real width() const {
      return right - left;
   }

   /// cutting value to fit into the interval
   real bound(const real &x) const {
      return math::bound(left, x, right);
   }

   /// relative distance from left point
   real relative(const real &x) const {
      return (x - left) / width();
   }

   /// test of interval emptiness
   bool isEmpty() const {
      if ((left == -INF) && (right == -INF)) return true;

      if ((left == INF) && (right == INF)) return true;

      return (right < left);
   }

   /// comparison operator
   bool operator==(const Interval I) const {
      // all empty intervals are equal
      if (this->isEmpty() && I.isEmpty()) return true;

      return (this->left == I.left) && (this->right == I.right);
   }

   /// comparison operator
   bool operator!=(const Interval I) const {
      return !(*this == I);
   }

   /// whether interval contains number x
   bool contains(const real x) const {
      return (this->left <= x) && (x <= this->right);
   }

   /// operator of number containment
   bool operator &&(const real x) const {
      return this->contains(x);
   }

   /// operator of interval and number intersection
   Interval operator &(const real x) const {
      if (*this && x) {
         return Interval(x, x);
      }
      else {
         return emptyInterval();
      }
   }

   /// whether interval contains another interval I
   bool contains(const Interval I) const {
      // empty interval is contained in every interval
      if (I.isEmpty()) return true;

      return (this->left <= I.left) && (I.right <= this->right);
   }

   /// whether two itervals have nonempty intersection
   bool intersects(const Interval I) const {
      return this->contains(I.left) || I.contains(this->left);
   }

   /// operator of nonempty intersection
   bool operator &&(const Interval I) const {
      return this->intersects(I);
   }

   /// operator of nonempty union
   bool operator ||(const Interval I) const {
      return !(this->isEmpty() && (I.isEmpty()));
   }

   /// intersection of two intervals
   Interval intersecion(const Interval I) const {
      if (!intersects(I)) return emptyInterval();

      return Interval(std::max(this->left, I.left), std::min(this->right, I.right));
   }

   /// operator of interval intersection
   Interval operator &(const Interval I) const {
      return this->intersecion(I);
   }

   /// size of interval
   real size() const {
      if (isEmpty()) return 0;

      return right - left;
   }

   /// random number from interval bounds
   real randomValue() const;

   /// ordering operator
   friend bool operator<(const Interval I, const Interval J);

   /// empty interval
   static Interval emptyInterval() {
      return Interval(1, -1);
   }

};

/// comparison operator
bool operator<(const Interval I, const Interval J);

/// output to stream
std::ostream& operator <<(std::ostream &os, const Interval I);

} // namespace math

} // namespace irafm

#endif // interval_h

// $Log: interval.h,v $
// Revision 1.4  2010/03/23 14:28:19  victor
// astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.3  2009/04/07 08:43:00  victor
// Astyle reformat
// Committed on the Free edition of March Hare Software CVSNT Server.
// Upgrade to CVS Suite for more features and support:
// http://march-hare.com/cvsnt/
//
// Revision 1.2  2007/11/07 12:52:34  victor
// Merging Interval implementation from TabuCones
//
// Revision 1.1  2007/04/05 14:14:26  Viktor
// Initial revision
//
